/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package core;

/**
 *
 * @author fotis
 */
public class Beam {

    private int id;
    private Node n1;
    private Node n2;
    private MaterialBeam beamMat;
    private MaterialSoil soilMat;
    private Section sec;
    private double L;
    private static double[][] K = new double[4][4];
    private static double[] F = new double[4];
    private int fTable[];
    private double[] Fext;
    private double[] Fint;
    private static final int NUM_OF_SEGS=30;
    private static double[] X=new double[NUM_OF_SEGS+1];
    private static double[] Y=new double[NUM_OF_SEGS+1];

    public Beam(int id, Node n1, Node n2,MaterialBeam beamMat,MaterialSoil soilMat,Section sec) {
        this.id = id;
        this.n1 = n1;
        this.n2 = n2;
        fTable = new int[4];
        for (int i = 0; i < 2; i++) {
            fTable[0 + i] = n1.getFTable()[i];
            fTable[2 + i] = n2.getFTable()[i];
        }
        L = n2.getX() - n1.getX();
        Fext = new double[2];
        Fint = new double[4];
        this.beamMat=beamMat;
        this.soilMat=soilMat;
        this.sec=sec;
    }
    public Beam(int id, Node n1, Node n2,MaterialBeam beamMat,MaterialSoil soilMat,Section sec,
            int fTable[],double[] Fext,double[] Fint) {
        this.id = id;
        this.n1 = n1;
        this.n2 = n2;
        this.fTable=fTable;
        this.Fext=Fext;
        this.Fint=Fint;
        this.beamMat=beamMat;
        this.soilMat=soilMat;
        this.sec=sec;
         L = n2.getX() - n1.getX();
    }

    public int getId() {
        return id;
    }

    public Node getN1() {
        return n1;
    }

    public Node getN2() {
        return n2;
    }

    public int[] getFTable() {
        return fTable;
    }

    public double[] getFext() {
        return Fext;
    }
    public double[] getF() {
        // Internal forces due to gravity forces
        double A=sec.getA();
        double Au=sec.getAu();
        double gammaSoil=soilMat.getGamma();
        double gammaConc=beamMat.getGamma();
        double p=L*(A*gammaConc+Au*gammaSoil);
        double pA=Fext[0]+p;
        double pB=Fext[1]+p;
        F[0]=+L/60*(21*pA+9*pB);
        F[1]=+L/60*(3*pA+2*pB)*L;
        F[2]=+L/60*(9*pA+21*pB);
        F[3]=-L/60*(2*pA+3*pB)*L;     
        return F;
    }

    public double[] getFint() {
        return Fint;
    }

    public static int getNumberOfPoints() {
        return NUM_OF_SEGS+1;
    }
    
    /**
     * Set beam section properties.
     * @param sec The section.
     */
    public void setBeamSec(Section sec) {
        this.sec = sec;
    }

    /**
     * Set external load (only vertical distributed load).
     * @param pA Load at the begining of the beam.
     * @param pB Load at the end of the beam.
     */
    public void setLoad(double pA, double pB) {
        Fext[0] = pA;
        Fext[1] = pB;

    }
     public double[] getX(){
        double x1=n1.getX();
        double x2=n2.getX();
        double dx=(x2-x1)/NUM_OF_SEGS;
        for(int i=0;i<NUM_OF_SEGS+1;i++){
            X[i]=x1+i*dx;
        }
        return X;
    }
   public double[] getU(){
        double u1=n1.getU()[0];
        double u2=n1.getU()[1];
        double u3=n2.getU()[0];
        double u4=n2.getU()[1];
        for(int i=0;i<NUM_OF_SEGS+1;i++){
            double x=X[i]-X[0];
            double N1=1-3*x*x/(L*L)+2*(x*x*x)/(L*L*L);
            double N2=x-2*x*x/L+(x*x*x)/(L*L);
            double N3=3*x*x/(L*L)-2.*(x*x*x)/(L*L*L);
            double N4=-x*x/L+(x*x*x)/(L*L);
            Y[i]=N1*u1+N2*u2+N3*u3+N4*u4;
        }
        return Y;
    }
   public double[] getM(){
        double MA=+Fint[1];
        double MB=-Fint[3];
        double dM=(MB-MA)/NUM_OF_SEGS;
        for(int i=0;i<NUM_OF_SEGS+1;i++){
            Y[i]=MA+i*dM;
        }    
        return Y;
   }
   public double[] getV(){
        double VA=-Fint[0];
        double VB=+Fint[2];
        double dV=(VB-VA)/NUM_OF_SEGS;
        for(int i=0;i<NUM_OF_SEGS+1;i++){
            Y[i]=VA+i*dV;
        }    
        return Y;
    }
   public double[] getP(){
       this.getU();
       for(int i=0;i<Y.length;i++){
           double Ks = soilMat.getKs()*sec.getB();
           Y[i]*=Ks;
       }
       return Y;
    }

    public double[][] getK() {
        double E = beamMat.getE();
        double A = sec.getA();
        double I = sec.getI();
        double Ks = soilMat.getKs()*sec.getB();
       
/*
        Ks*= ((L * L * L) / 420.);
        double K02 = 2. * E * I / L;
        double K04 = 4. * E * I / L;
        double K06 = 6. * E * I / (L * L);
        double K12 = 12. * E * I / (L * L * L);

        K[0][0] = +K12 + Ks*156 / (L * L);
        K[0][1] = +K06 + Ks*22 / L;
        K[0][2] = -K12 + Ks*54 / (L * L);
        K[0][3] = +K06 - Ks*13 / L;

        K[1][0] = +K06 + Ks*22 / L;
        K[1][1] = +K04 + Ks*4;
        K[1][2] = -K06 + Ks*13 / L;
        K[1][3] = +K02 - Ks*3;

        K[2][0] = -K12 + Ks*54 / (L * L);
        K[2][1] = -K06 + Ks*13 / L;
        K[2][2] = +K12 + Ks*156 / (L * L);
        K[2][3] = -K06 - 22 / L;

        K[3][0] = +K06 - Ks*13 / L;
        K[3][1] = +K02 - Ks*3;
        K[3][2] = -K06 - Ks*22 / L;
        K[3][3] = +K04 + Ks*4;
        return K;
*/
  
  /*     
        Ks = soilMat.getKs()*sec.getB();
        K[0][0] = +K12 + Ks;
        K[0][1] = +K06;
        K[0][2] = -K12;
        K[0][3] = +K06;

        K[1][0] = +K06;
        K[1][1] = +K04 + Ks;
        K[1][2] = -K06;
        K[1][3] = +K02;

        K[2][0] = -K12;
        K[2][1] = -K06;
        K[2][2] = +K12 + Ks;
        K[2][3] = -K06;

        K[3][0] = +K06;
        K[3][1] = +K02;
        K[3][2] = -K06;
        K[3][3] = +K04 + Ks;
   
        return K;
  */   
 
    double lambda=L*Math.pow(Ks/(4*E*I), 0.25);
    double sn=Math.sin(lambda);
    double cs=Math.cos(lambda);
    double sh=Math.sinh(lambda);
    double ch=Math.cosh(lambda);
    double dl=lambda/L;
    double a1=sn*cs-sh*ch;
    double a2=-2*dl*dl*(sn*cs+sh*ch);
    double a3=sh*cs-ch*sn;
    double a4=dl*(sn*sn+sh*sh);
    double a5=2*dl*(sn*sh);
    double a6=2*dl*dl*(sh*cs+ch*sn);
    
    double K0=2*E*I*lambda/(L*(sn*sn-sh*sh));
    K[0][0] = +K0*a2;
    K[0][1] = -K0*a4;
    K[0][2] = +K0*a6;
    K[0][3] = -K0*a5;

    K[1][0] = -K0*a4;
    K[1][1] = +K0*a1;
    K[1][2] = +K0*a5;
    K[1][3] = +K0*a3;

    K[2][0] = +K0*a6;
    K[2][1] = +K0*a5;
    K[2][2] = +K0*a2;
    K[2][3] = +K0*a4;

    K[3][0] = -K0*a5;
    K[3][1] = +K0*a3;
    K[3][2] = +K0*a4;
    K[3][3] = +K0*a1;
    
    return K;
    }

    public double[][] getKb() {
        double E = beamMat.getE();
        double A = sec.getA();
        double I = sec.getI();

        double K02 = 2. * E * I / L;
        double K04 = 4. * E * I / L;
        double K06 = 6. * E * I / (L * L);
        double K12 = 12. * E * I / (L * L * L);

        K[0][0] = +K12;
        K[0][1] = +K06;
        K[0][2] = -K12;
        K[0][3] = +K06;

        K[1][0] = +K06;
        K[1][1] = +K04;
        K[1][2] = -K06;
        K[1][3] = +K02;

        K[2][0] = -K12;
        K[2][1] = -K06;
        K[2][2] = +K12;
        K[2][3] = -K06;

        K[3][0] = +K06;
        K[3][1] = +K02;
        K[3][2] = -K06;
        K[3][3] = +K04;
        return K;
    }

    public void update() {
        double KL[][]=getK();
        F=this.getF();

        double u1=n1.getU()[0];
        double u2=n1.getU()[1];
        double u3=n2.getU()[0];
        double u4=n2.getU()[1];
       
        Fint[0]=KL[0][0]*u1+KL[0][1]*u2+KL[0][2]*u3+KL[0][3]*u4-F[0];
        Fint[1]=KL[1][0]*u1+KL[1][1]*u2+KL[1][2]*u3+KL[1][3]*u4-F[1];
        Fint[2]=KL[2][0]*u1+KL[2][1]*u2+KL[2][2]*u3+KL[2][3]*u4-F[2];
        Fint[3]=KL[3][0]*u1+KL[3][1]*u2+KL[3][2]*u3+KL[3][3]*u4-F[3];
         
    }
    
    public void clear(){
        Fint[0]=0.;
        Fint[1]=0.;
        Fint[2]=0.;
        Fint[3]=0.;
    }

    @Override
    public String toString() {
        String s="\t<beam>\n";
        s+="\t\t<id>"+id+"</id>\n";
        s+="\t\t<n1>"+n1.getId()+"</n1>\n";
        s+="\t\t<n2>"+n2.getId()+"</n2>\n";
        s+="\t\t<L>"+L+"</L>\n";
        s+="\t\t<fTable>"+fTable[0]+" "+fTable[1]+" "+fTable[2]+" "+fTable[3]+" "+"</fTable>\n";
        s+="\t\t<Fext>"+Fext[0]+" "+Fext[1]+"</Fext>\n"; 
        s+="\t\t<Fint>"+Fint[0]+" "+Fint[1]+" "+Fint[2]+" "+Fint[3]+"</Fint>\n";
        s+="\t</beam>\n";
        return s;
    }
}
