/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package core;

import java.text.DecimalFormat;
import java.util.Vector;

/**
 *
 * @author fotis
 */
public class Domain {

    private Vector<Node> nodes;
    private Vector<Beam> beams;
    private MaterialSoil soilMat;
    private MaterialBeam beamMat;
    private Section beamSec;
    private Soe soe;
    private double L;
    private double[] xArray;
    private double[] yArray;
    private String project;
    private String user;
    private String comments;

    public enum ResultsType {

        U, V, M, P
    };

    private ResultsType resultsType ;

    public Domain() {
        nodes = new Vector<Node>();
        beams = new Vector<Beam>();
        this.reset();
        soe = new Soe();
        L = 0.;
        xArray = new double[0];
        yArray = new double[0];
        resultsType = ResultsType.U;
        project=new String();
        user=new String();
        comments= new String();
    }

    public Vector<Node> getNodes() {
        return nodes;
    }

    public Vector<Beam> getBeams() {
        return beams;
    }

    public MaterialBeam getBeamMat() {
        return beamMat;
    }

    public Section getBeamSec() {
        return beamSec;
    }

    public MaterialSoil getSoilMat() {
        return soilMat;
    }

    public double getL() {
        return L;
    }

    public void setBeamMat(double gamma, double E, double nu) {
        beamMat.setGamma(gamma);
        beamMat.setE(E);
        beamMat.setNu(nu);
    }

    public void setSoilMat(double gamma, double Ks) {
        soilMat.setGamma(gamma);
        soilMat.setKs(Ks);
    }

    public void setBeamSec(Section beamSec) {
        this.beamSec = beamSec;
        for (int i = 0; i < beams.size(); i++) {
            beams.get(i).setBeamSec(beamSec);
        }
    }

    public String getComments() {
        return comments;
    }

    public void setComments(String comments) {
        this.comments = comments;
    }

    public String getProject() {
        return project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    /**
     * Set domain nodes.
     * The elements and all loads are also cleared.
     * The soe is initialized.
     * The length is found.
     * @param nodes
     */
    public void setNodes(Vector<Node> nodes) {
        this.nodes = nodes;
        // Create beams
        beams.removeAllElements();
        for (int i = 0; i < nodes.size() - 1; i++) {
            beams.add(new Beam(i + 1, nodes.get(i), nodes.get(i + 1),beamMat,soilMat,beamSec));
        }
        // Set length
        if (nodes.size() > 1) {
            L = nodes.get(nodes.size() - 1).getX() - nodes.get(0).getX();
        }
        // Set xArray/yArray
        xArray = new double[nodes.size()];
        yArray = new double[nodes.size()];
        // Init xArray
        for (int i = 0; i < nodes.size(); i++) {
            xArray[i] = nodes.get(i).getX();
        }
    }
    public void addNode(Node n){
        nodes.add(n);
        // Set length
        if (nodes.size() > 1) {
            L = nodes.get(nodes.size() - 1).getX() - nodes.get(0).getX();
        }
        // Set xArray/yArray
        xArray = new double[nodes.size()];
        yArray = new double[nodes.size()];
        // Init xArray
        for (int i = 0; i < nodes.size(); i++) {
            xArray[i] = nodes.get(i).getX();
        }
    }
    public void addBeam(Beam b){
        beams.add(b);
    }    
    public void setResultsType(ResultsType resultsType) {
        this.resultsType = resultsType;
    }

    public double[] getX() {
        double[] x = new double[nodes.size()];
        for (int i = 0; i < nodes.size(); i++) {
            x[i] = nodes.get(i).getX();
        }
        return x;
    }

    public double[] getX(int i) {
        return beams.get(i).getX();
    }

    public double[] getY(int i) {
        double[] y = new double[0];

        switch (resultsType) {
            case U:
                y = beams.get(i).getU();
                break;
            case M:
                y = beams.get(i).getM();
                break;
            case V:
                y = beams.get(i).getV();
                break;
            case P:
                y = beams.get(i).getP();
                break;
            default:
        }
        return y;
    }

    public double[] getMaxMin() {
        double[] ret = new double[5];
        for (int i = 0; i < beams.size(); i++) {
            // Get x/y
            double[] x = getX(i);
            double[] y = getY(i);
            // loop
            for (int j = 0; j < y.length; j++) {
                // max
                if (y[j] > ret[1]) {
                    ret[1] = y[j];
                    ret[2] = x[j];
                }
                // min
                if (y[j] < ret[3]) {
                    ret[3] = y[j];
                    ret[4] = x[j];
                }
            }
            // max/min
            ret[0] = Math.max(Math.abs(ret[1]), Math.abs(ret[3]));
        }
        return ret;
    }
    public Object[][] getTable() {
        DecimalFormat f1=new DecimalFormat("0.00000000");
        DecimalFormat f2=new DecimalFormat("0.000");
        Object[][] table = new Object[2*beams.size()][7];
        int k=0;
        for (int i = 0; i < beams.size(); i++) {
            double[] uA=nodes.get(i).getU();
            double[] uB=nodes.get(i+1).getU();
            double[] F=beams.get(i).getFint();
            
            table[k][0]=""+(i+1);
            table[k][1]=""+(i+1);
            table[k][2]=f1.format(uA[0]);
            table[k][3]=f1.format(uA[1]);
            table[k][6]=f2.format(soilMat.getKs()*uA[0]);
            table[k][4]=f2.format(+F[1]);
            table[k][5]=f2.format(-F[0]);
            k++;
            
            table[k][0]="";
            table[k][1]=""+(i+2);
            table[k][2]=f1.format(uB[0]);
            table[k][3]=f1.format(uB[1]);
            table[k][6]=f2.format(soilMat.getKs()*uB[0]);
            table[k][4]=f2.format(-F[3]);
            table[k][5]=f2.format(+F[2]);
            k++;
        }
        return table;
    }

    /**
     * Resets the domain to default values.
     */
    public void reset() {
        soilMat = new MaterialSoil(18., 28000.);
        beamMat =
                new MaterialBeam(20., 21000000., 0.15);
        beamSec =
                new SectionTrapezoidal(2.0, 2.0, 0.5, 0.5, 0.5, 0.5);
        nodes.removeAllElements();
        beams.removeAllElements();
    }
    /**
     * Clears results.
     */
    public void clear() {
        // Update nodes
        for (int i = 0; i < nodes.size(); i++) {
            nodes.get(i).clear();
        }
        for (int i = 0; i < beams.size(); i++) {
            beams.get(i).clear();
        }
    }

    public void solve() {
        soe.init(2 * nodes.size());
        // Add elements
        for (int i = 0; i < beams.size(); i++) {
            soe.addMatrix(beams.get(i).getK(), beams.get(i).getFTable());
        }
        // Add elemental loads
        for (int i = 0; i < beams.size(); i++) {
            soe.addVector(beams.get(i).getF(), beams.get(i).getFTable());
        }
        // Add nodal loads
        for (int i = 0; i < nodes.size(); i++) {
            soe.addVector(nodes.get(i).getF(), nodes.get(i).getFTable());
        }
        // Solve
        //soe.print();
        soe.solve();
        // Update nodes
        for (int i = 0; i < nodes.size(); i++) {
            nodes.get(i).update(soe.getX());
        }
        // Update beams
        for (int i = 0; i < beams.size(); i++) {
            beams.get(i).update();
        }
        this.getTable();
    }
    
    @Override
    public String toString(){
        String s="<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
        s+="<jWinkler>\n";
        // settings
        s+="\t<settings>\n";
        s+="\t\t<project>"+project+" </project>\n";
        s+="\t\t<user>"+user+" </user>\n";
        s+="\t\t<comments>"+comments+" </comments>\n";
//        s+="\t\t<date>"+date+" </date>\n";
//        s+="\t\t<time>"+time+" </time>\n";
        s+="\t</settings>\n";
        // soil
        s+=soilMat.toString();
        // concrete
        s+=beamMat.toString();
        // section
        s+=beamSec.toString();
        // nodes
        for(int i=0;i<nodes.size();i++){
            s+=nodes.get(i).toString();
        }
        // beams
        for(int i=0;i<beams.size();i++){
            s+=beams.get(i).toString();
        }
        s+="</jWinkler>\n";
        return s;        
    }
}
