/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core;

import java.awt.Color;
import java.awt.Graphics2D;
import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 *
 * @author fotis
 */
public class SectionTrapezoidal extends Section {

    private double b0;
    private double h0;
    private double h1;
    private double h2;

    public SectionTrapezoidal(double b, double h, double b0, double h0, double h1,double h2) {
        super(b, h);
        tag="TRPZ";
        
        // data
        this.b0 = b0;
        this.h0 = h0;
        this.h1 = h1;
        this.h2 = h2;
        
        // lower rectangle 
	double A1=b*h0;
	double ys1=0.5*h0;
	double I1=b*h0*h0*h0/12;

        // middle trapezoid
	double A2=0.5*(b+b0)*h1;
	double ys2=h0+0.5*h1;
	double I2=(h1*h1*h1*(b*b+4*b*b0+b0*b0))/(36*(b+b0));

        // lower rectangle
	double A3=b0*h2;
	double ys3=h0+h1+0.5*h2;
	double I3=b0*h2*h2*h2/12;

        // area
	A=A1+A2+A3;
        
        // moment of inertia
	double ys=(ys1*A1+ys2*A2+ys3*A3)/(A1+A2+A3);
	I=I1+(ys-ys1)*(ys-ys1)*A1+
	  I2+(ys-ys2)*(ys-ys2)*A2+
	  I3+(ys-ys3)*(ys-ys3)*A3;
        
        // soil area
        Au=b*h-A;
    }
    
    public void draw(Graphics2D g2,int x0,int y0) {
        g2.setColor(Color.BLACK);
        g2.drawLine(x0, y0 - 20, x0 + 210, y0 - 20);
        g2.drawLine(x0, y0 - 115, x0 + 210, y0 - 115);
        g2.drawLine(x0, y0 - 20, x0, y0 - 115);
        g2.drawLine(x0 + 210, y0 - 20, x0 + 210, y0 - 115);
        g2.setColor(Color.BLUE);
        g2.drawLine(x0 + 10, y0 - 40, x0 + 85, y0 - 40);
        g2.drawLine(x0 + 10, y0 - 40, x0 + 10, y0 - 65);
        g2.drawLine(x0 + 85, y0 - 40, x0 + 85, y0 - 65);
        g2.drawLine(x0 + 10, y0 - 65, x0 + 35, y0 - 75);
        g2.drawLine(x0 + 85, y0 - 65, x0 + 60, y0 - 75);
        g2.drawLine(x0 + 35, y0 - 75, x0 + 35, y0 - 100);
        g2.drawLine(x0 + 60, y0 - 75, x0 + 60, y0 - 100);
        g2.drawLine(x0 + 35, y0 - 100, x0 + 60, y0 - 100);
        g2.setColor(Color.BLACK);
        NumberFormat f=new DecimalFormat("###0.00");
        g2.drawString("b ", x0+100, y0-100);
        g2.drawString("b0", x0+100, y0-85);
        g2.drawString("h ", x0+100, y0-70);
        g2.drawString("h0", x0+100, y0-55);
        g2.drawString("h1", x0+100, y0-40);
        g2.drawString("h2", x0+100, y0-25);
        g2.drawString(" = "+f.format(b)+"m", x0+120, y0-100);
        g2.drawString(" = "+f.format(b0)+"m", x0+120, y0-85);
        g2.drawString(" = "+f.format(h)+"m", x0+120, y0-70);
        g2.drawString(" = "+f.format(h0)+"m", x0+120, y0-55);
        g2.drawString(" = "+f.format(h1)+"m", x0+120, y0-40);
        g2.drawString(" = "+f.format(h2)+"m", x0+120, y0-25);
   }
   
    @Override
    public String toString(){
        String s="\t<section_trpz>\n";
        s+="\t\t<b>"+b+"</b>\n";
        s+="\t\t<h>"+h+"</h>\n";
        s+="\t\t<b0>"+b0+"</b0>\n";
        s+="\t\t<h0>"+h0+"</h0>\n";
        s+="\t\t<h1>"+h1+"</h1>\n";
        s+="\t\t<h2>"+h2+"</h2>\n";
        s+="\t\t<A>"+A+"</A>\n";
        s+="\t\t<I>"+I+"</I>\n";
        s+="\t</section_trpz>\n";
        return s;
    }    
}
