package core;

import java.text.DecimalFormat;

/**
 * Soe - example of calling the Java version of Dsygv (from LAPACK).
 *
 * To compile and run:
 *
 * # javac -classpath .:f2jutil.jar:blas.jar:lapack.jar Soe.java
 * # java -classpath .:f2jutil.jar:blas.jar:lapack.jar:xerbla.jar Soe
 * on return info = 0
 *
 **/
public class Soe {

    private double[][] A;
    private double[] B;
    private double[] X;
    private int size;

    public Soe() {
    }

    public void init(int n) {
        size = n;
        A = new double[n][n];
        B = new double[n];
        X = new double[n];
    }

    public void clearB() {
        for (int i = 0; i < B.length; i++) {
            B[i] = 0.;
        }
    }

    public void addMatrix(double[][] m, int fTable[]) {
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                A[fTable[i]][fTable[j]] += m[i][j];
            }
        }
    }

    public void addVector(double v[], int fTable[]) {
        for (int i = 0; i < fTable.length; i++) {
            B[fTable[i]] += v[i];
        }
    }

    public double[] getX() {
        return X;
    }

    public void decompose() {
    }

    public void substitute() {

    }

    public void solve() {
        for(int i=0;i<size;i++){
            X[i]=B[i];
        }
        // LU decomposition without pivoting 
        for (int k = 0; k < size - 1; k++) {
            //if (temp[k][k] == 0) exit(1);//error("pivot is zero in Matrix::GaussElimNP()");
            for (int i = k + 1; i < size; i++) {
                if (A[i][k] != 0) {
                    double mult = A[i][k] / A[k][k];
                    A[i][k] = mult;
                    for (int j = k + 1; j < size; j++) {
                        A[i][j] -= mult * A[k][j];
                    }
                }
            }
        }
        // forward substitution for L.y = b.y still stored in b
        for (int i = 1; i < size; i++) {
            for (int j = 0; j < i; j++) {
                X[i] -= A[i][j] * X[j];
            }
        }
        // back substitution for U.x = y. x still stored in b
        for (int i = size - 1; i >= 0; i--) {
            for (int j = i + 1; j < size; j++) {
                X[i] -= A[i][j] * X[j];
            }
            X[i] /= A[i][i];
        }
    }

    void print() {
        DecimalFormat f = new DecimalFormat("##0.00");
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                System.out.print(f.format(A[i][j]) + "\t");
            }
            System.out.print(" | " + f.format(B[i]) + "\n");
        }
    }
    void printX() {
        DecimalFormat f = new DecimalFormat("##0.000000000");
        for (int i = 0; i < size; i++) {
            System.out.println(f.format(X[i]));
        }
    }
}
