/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gui;

import core.Domain;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import javax.swing.*;

/**
 *
 * @author fotis
 */
public class DrawPanel extends JPanel{
    
    protected double[] xArray;
    protected double[] yArray;
    protected double L;
    protected Domain domain;
    
    /**
        These fields determine where to put the origo of the
	coordinate system.
    */

    protected double originX;
    protected double originY;

    /** 
	These fields determine how many pixels (on the screen)
	corresponds to une in the (user) coordinate system.
    */

    protected double resolution_x;
    protected double resolution_y;
    

    
    public DrawPanel() {
        xArray=new double[0]; 
    }

    public void setDomain(Domain domain) {
        this.domain = domain;
    }

    public void init() {
        this.xArray = domain.getX();
        yArray=new double[xArray.length];
        L=xArray[xArray.length-1]-xArray[0];
    }
        
    public void plot(double[] yArray){
        this.yArray=yArray;
        repaint();
    }
    
    protected void drawLine(Graphics2D g2,double x1,double y1,double x2,double y2,int thick){
        double xA=resolution_x+x1*resolution_x;
        double yA=getHeight()/2+y1*resolution_y;
        double xB=resolution_x+x2*resolution_x;
        double yB=getHeight()/2+y2*resolution_y;
        //g2.setColor(Color.BLUE);
        g2.setStroke(new BasicStroke(thick)); 
        g2.draw(new Line2D.Double(xA,yA,xB,yB));
    }
    protected void drawString(Graphics2D g2,String s,double x1,double y1,Color c){
        float xA=(float) (resolution_x + x1 * resolution_x);
        float yA=(float) (getHeight() / 2 + y1 * resolution_y);
        g2.setColor(c);
        g2.drawString(s,xA,yA);
    }
    protected void drawPath(Graphics2D g2,double[] x,double[] y,Color c){
        if(y.length>0){
            g2.setColor(c);
            GeneralPath path=new GeneralPath();
            double xC=resolution_x+x[0]*resolution_x;
            double yC=getHeight()/2+y[0]*resolution_y;
            path.moveTo(xC, yC);
            for(int i=1;i<x.length;i++){
                xC=resolution_x+x[i]*resolution_x;
                yC=getHeight()/2+y[i]*resolution_y;
               path.lineTo(xC, yC);
            }
            g2.draw(path);
        }
    }
    protected void drawCircle(Graphics2D g2,double x0,double y0,double r){
        double xC=resolution_x+x0*resolution_x;
        double yC=getHeight()/2+y0*resolution_y;
        g2.setColor(Color.BLUE);
        g2.setStroke(new BasicStroke(1));
        g2.fill(new Ellipse2D.Double(xC-0.5*r,yC-0.5*r,r,r));
    }
    protected void drawBeam(Graphics2D g2){    
        double xA = xArray[0];
        double xB = xArray[xArray.length - 1];
        drawLine(g2, xA, 0., xB, 0., 2);
        for (int i = 0; i < xArray.length; i++) {
            drawCircle(g2, xArray[i], 0, 8);
        }
    }
    protected void drawCoordinateSystem(Graphics2D g2, int width, int height) {
	g2.setStroke(new BasicStroke(1)); 
	g2.setColor(Color.LIGHT_GRAY);
	g2.draw(new Line2D.Double(0, originY, width,originY));
	// draw ticks
	g2.setColor(Color.DARK_GRAY);
	int mark_x = 0;
        for (double x = originX; x < width; x += resolution_x) {
            g2.draw(new Line2D.Double(x, 0, x, 5));
            g2.draw(new Line2D.Double(x, height - 5, x, height));
            g2.drawString("" + mark_x, (float) x, (float) height - 10);
            mark_x++;
        }
    }
}
