/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Vector;

/**
 *
 * @author fotis
 */
public class DrawPanelInput extends DrawPanel{

    public DrawPanelInput() {
        super();
    }

    public void plot() {
        repaint();
    }
    protected void drawSprings(Graphics2D g2){
        double springX[]={+0,+0,+3,-3, +3, -3, +3, -3, +3, -3,  0, -5, +5};
        double springY[]={+0,+3,+6,+9,+12,+15,+18,+21,+24,+27,+30,+30,+30};
        int height = this.getHeight();
        g2.setColor(Color.GRAY);
        for(int k=0;k<domain.getNodes().size();k++) {
            double x0=1.+domain.getNodes().get(k).getX();
            double y0=0.;
            double xC=resolution_x*x0;
            double yC=0.5*height+y0;
            for(int i=0;i<springX.length-1;i++) {
                double x1=xC+springX[i  ];
                double y1=yC+springY[i  ];
                double x2=xC+springX[i+1];
                double y2=yC+springY[i+1];
                g2.draw(new Line2D.Double(x1,y1,x2,y2));
            }
        }
    }
    protected void drawLoads(Graphics2D g2){
        NumberFormat f=new DecimalFormat("###0.0");
        g2.setColor(Color.RED);
        int height = this.getHeight();
        // Nodal loads
        Vector<core.Node> nodes=domain.getNodes();
        for(int i=0;i<nodes.size();i++) {
            double X=nodes.get(i).getX()+1.;
            double V=nodes.get(i).getF()[0];
            double M=nodes.get(i).getF()[1];
            double xC=resolution_x*X;
            double yC=0.5*height;
            if(V!=0.){
                g2.draw(new Line2D.Double(xC,yC-40,xC,yC-4));
                g2.draw(new Line2D.Double(xC-3,yC-9,xC,yC-4));
                g2.draw(new Line2D.Double(xC+3,yC-9,xC,yC-4));
                g2.drawString(new String(f.format(V)), (float)xC, (float)(yC-40));
            }
             if(M!=0.){
                g2.draw(new Arc2D.Double(xC-30,yC-30,60,60,45,90,Arc2D.OPEN));
                g2.draw(new Line2D.Double(xC,yC,xC-21.213,yC-21.2132));
                g2.draw(new Line2D.Double(xC+21.2132,yC-21.2132,xC+15.4537,yC-21.2132));
                g2.draw(new Line2D.Double(xC+21.2132,yC-21.2132,xC+21.2132,yC-26.2132));
                g2.drawString(new String(f.format(M)), (float)(xC+22.2132), (float)(yC-25));
            }
        }
        // Beam loads
        Vector<core.Beam> beams=domain.getBeams();
        for(int i=0;i<beams.size();i++) {
            double x1=(beams.get(i).getN1().getX()+1.)*resolution_x;
            double x2=(beams.get(i).getN2().getX()+1.)*resolution_x;
            double pA=beams.get(i).getFext()[0];
            double pB=beams.get(i).getFext()[1];   
            double y0=0.5*height;
            if(pA!=0. || pB!=0.){
                drawArrow(g2,x1,40);
                g2.drawString(new String(f.format(pA)), (float)(x1), (float)(y0-42));
                int nArrows=(int) ((x2 - x1) / 10);
                double dx=(x2 - x1)/nArrows;
                for(int j=0;j<nArrows;j++){
                    double xA=x1+j*dx;
                    double hA=40-20/(x2-x1)*j*dx;
                    drawArrow(g2,xA,hA);
                }
                drawArrow(g2,x2,20);
                g2.drawString(new String(f.format(pB)), (float)(x2), (float)(y0-22)); 
                g2.draw(new Line2D.Double(x1,y0-40,x2,y0-20));
            }
        }
        
    }
    protected void drawArrow(Graphics2D g2,double x,double h){
        double y=0.5*this.getHeight();
        g2.draw(new Line2D.Double(x,y-h,x,y-4));
        g2.draw(new Line2D.Double(x-3,y-9,x,y-4));
        g2.draw(new Line2D.Double(x+3,y-9,x,y-4));        
    }
    
    @Override
     protected void drawBeam(Graphics2D g2){    
        g2.setColor(Color.BLUE);
        int height = this.getHeight();
        Vector<core.Node> nodes=domain.getNodes();
        double xA = nodes.get(0).getX();
        double xB = nodes.get(nodes.size()-1).getX();
        drawLine(g2, xA, 0., xB, 0., 2);
        for(int i=0;i<nodes.size();i++) {
            double X=nodes.get(i).getX()+1.;
            double xC=resolution_x*X;
            double yC=0.5*height;
            drawCircle(g2, nodes.get(i).getX(), 0, 8);
            g2.drawString(new String(""+(i+1)), (float)(xC+5), (float)(yC+11));
        }
        for(int i=0;i<nodes.size()-1;i++) {
            double X1=nodes.get(i).getX()+1.;
            double X2=nodes.get(i+1).getX()+1.;
            double xC=0.5*resolution_x*(X1+X2);
            double yC=0.5*height;
            drawCircle(g2, nodes.get(i).getX(), 0, 8);
            //g2.drawString(new String("("+(i+1)+")"), (float)(xC), (float)(yC+11));
        }
    }

    @Override
    public void paintComponent(Graphics g) {
	// Define graphics
        Graphics2D g2 = (Graphics2D) g;
        // Get dimensions
	int width  = this.getWidth();
	int height = this.getHeight();
	// Clear window
	g2.setBackground(Color.WHITE);
	g2.clearRect(0, 0, width, height);
        // Check if domain is ok (todo)
        if(domain==null)
            return;
        // Get nodes
        Vector<core.Node> nodes=domain.getNodes();
        // Draw
        if(domain.getNodes().size()>1){          
            resolution_x=width/(L+2);
            resolution_y=resolution_x;
            originX=resolution_x;
            originY=height/2;     
            drawCoordinateSystem(g2, width, height);
            drawSprings(g2);
            drawLoads(g2);
            drawBeam(g2);
        }
        if(resolution_x==0){
            domain.getBeamSec().draw(g2,50,this.getHeight()-15);
            domain.getSoilMat().draw(g2,270,this.getHeight()-15);
            domain.getBeamMat().draw(g2,270,this.getHeight()-15);
        }else{
            domain.getBeamSec().draw(g2,(int)resolution_x,this.getHeight()-15);
            domain.getSoilMat().draw(g2,(int)resolution_x+220,this.getHeight()-15);
            domain.getBeamMat().draw(g2,(int)resolution_x+220,this.getHeight()-15);
        }
        //drawMaterials(g2);
    }
}
