package misc;

import java.io.File;
import java.io.IOException;
import org.w3c.dom.Document;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class XMLParser {

    public XMLParser() {
    }

    private int parseInt(Node node, String name) {
        Element element = (Element) node;
        NodeList list = element.getElementsByTagName(name);
        Element idElement = (Element) list.item(0);
        NodeList idText = idElement.getChildNodes();
        String s = ((Node) idText.item(0)).getNodeValue();
        return Integer.parseInt(s);
    }

    private double parseDouble(Node node, String name) {
        Element element = (Element) node;
        NodeList list = element.getElementsByTagName(name);
        Element idElement = (Element) list.item(0);
        NodeList idText = idElement.getChildNodes();
        String s = ((Node) idText.item(0)).getNodeValue();
        return Double.parseDouble(s);
    }

    private int[] parseIntArray(Node node, String name) {
        Element element = (Element) node;
        NodeList list = element.getElementsByTagName(name);
        Element idElement = (Element) list.item(0);
        NodeList idText = idElement.getChildNodes();
        String[] s = ((Node) idText.item(0)).getNodeValue().split(" ");
        int array[] = new int[s.length];
        for (int i = 0; i < s.length; i++) {
            array[i] = Integer.parseInt(s[i]);
        }
        return array;
    }

    private double[] parseDoubleArray(Node node, String name) {
        Element element = (Element) node;
        NodeList list = element.getElementsByTagName(name);
        Element idElement = (Element) list.item(0);
        NodeList idText = idElement.getChildNodes();
        String[] s = ((Node) idText.item(0)).getNodeValue().split(" ");
        double array[] = new double[s.length];
        for (int i = 0; i < s.length; i++) {
            array[i] = Double.parseDouble(s[i]);
        }
        return array;
    }

    private String parseString(Node node, String name) {
        Element element = (Element) node;
        NodeList list = element.getElementsByTagName(name);
        Element idElement = (Element) list.item(0);
        String s = "";
        if (idElement != null) {
            NodeList idText = idElement.getChildNodes();
            s = ((Node) idText.item(0)).getNodeValue().trim();
        }
        return s;
    }

    public void parse(core.Domain domain, File file) throws IOException {
        try {
            domain.reset();
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            Document doc = docBuilder.parse(file);

            // normalize text representation
            doc.getDocumentElement().normalize();
            System.out.println("Root element of the doc is " +
                    doc.getDocumentElement().getNodeName());


            NodeList nodes;

            // General
            nodes = doc.getElementsByTagName("settings");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    String project = parseString(node, "project");
                    String user = parseString(node, "user");
                    String comments = parseString(node, "comments");
                    //String date = parseString(node, "date");
                    //String time = parseString(node, "time");

                    // create 
                    //System.out.println("project  : " + project);
                    //System.out.println("user     : " + user);
                    //System.out.println("comments : " + comments);
                    //System.out.println("date     : " + date);
                    //System.out.println("time     : " + time);
                    domain.setProject(project);
                    domain.setUser(user);
                    domain.setComments(comments);
                }
            }

            // Soil
            nodes = doc.getElementsByTagName("soil");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    double Ks = parseDouble(node, "Ks");
                    double gamma = parseDouble(node, "gamma");

                    // create 
                    //System.out.println("Ks     : " + Ks);
                    //System.out.println("gamma  : " + gamma);
                    domain.setSoilMat(gamma, Ks);
                }
            }

            // Concrete
            nodes = doc.getElementsByTagName("concrete");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    double E = parseDouble(node, "E");
                    double nu = parseDouble(node, "nu");
                    double gamma = parseDouble(node, "gamma");

                    // create 
                    //System.out.println("E      : " + E);
                    //System.out.println("nu     : " + nu);
                    //System.out.println("gamma  : " + gamma);
                    domain.setBeamMat(gamma, E, nu);
                }
            }

            // Section trapezoidal
            nodes = doc.getElementsByTagName("section_trpz");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    double b = parseDouble(node, "b");
                    double h = parseDouble(node, "h");
                    double b0 = parseDouble(node, "b0");
                    double h0 = parseDouble(node, "h0");
                    double h1 = parseDouble(node, "h1");
                    double h2 = parseDouble(node, "h2");
                    double A = parseDouble(node, "A");
                    double I = parseDouble(node, "I");

                    // create 
                    //System.out.println("b      : " + b);
                    //System.out.println("h      : " + h);
                    //System.out.println("b0     : " + b0);
                    //System.out.println("h0     : " + h0);
                    //System.out.println("h1     : " + h1);
                    //System.out.println("h2     : " + h2);
                    //System.out.println("A      : " + A);
                    //System.out.println("I      : " + I);
                    core.SectionTrapezoidal sec=new core.SectionTrapezoidal(b, h, b0, h0, h1, h2);
                    domain.setBeamSec(sec);               
                }
            }

            // Section rectangular
            nodes = doc.getElementsByTagName("section_rect");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    double b = parseDouble(node, "b");
                    double h = parseDouble(node, "h");
                    double b0 = parseDouble(node, "b0");
                    double h0 = parseDouble(node, "h0");
                    double h1 = parseDouble(node, "h1");
                    double A = parseDouble(node, "A");
                    double I = parseDouble(node, "I");

                    // create 
                    //System.out.println("b      : " + b);
                    //System.out.println("h      : " + h);
                    //System.out.println("b0     : " + b0);
                    //System.out.println("h0     : " + h0);
                    //System.out.println("h1     : " + h1);
                    //System.out.println("A      : " + A);
                    //System.out.println("I      : " + I);
                    core.SectionRectangle sec=new core.SectionRectangle(b, h, b0, h0, h1);
                    domain.setBeamSec(sec);                     
                }
            }

            // Section user
            nodes = doc.getElementsByTagName("section_user");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    double b = parseDouble(node, "b");
                    double h = parseDouble(node, "h");
                    double A = parseDouble(node, "A");
                    double I = parseDouble(node, "I");

                    // create 
                    //System.out.println("b      : " + b);
                    //System.out.println("h      : " + h);
                    //System.out.println("A      : " + A);
                    //System.out.println("I      : " + I);
                    core.SectionUser sec=new core.SectionUser(b, h, A, I);
                    domain.setBeamSec(sec);
                }
            }

            // Nodes
            nodes = doc.getElementsByTagName("node");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    int id = parseInt(node, "id");
                    double x = parseDouble(node, "x");
                    double u[] = parseDoubleArray(node, "u");
                    int fTable[] = parseIntArray(node, "fTable");
                    double F[] = parseDoubleArray(node, "F");
                    // create 
                    //System.out.println("id     : " + id);
                    //System.out.println("x      : " + x);
                    //System.out.println("u      : " + u[0] + "  " + u[1]);
                    //System.out.println("fTable : " + fTable[0] + "  " + fTable[1]);
                    //System.out.println("F      : " + F[0] + "  " + F[1]);
                    core.Node n=new core.Node(id, x, fTable, F, u);
                    domain.addNode(n);
                }
                
            }


            // Beams
            nodes = doc.getElementsByTagName("beam");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    // parse
                    int n1 = parseInt(node, "n1");
                    int n2 = parseInt(node, "n2");
                    double L = parseDouble(node, "L");
                    int fTable[] = parseIntArray(node, "fTable");
                    double Fext[] = parseDoubleArray(node, "Fext");
                    double Fint[] = parseDoubleArray(node, "Fint");

                    // create 
                    //System.out.println("n1     : " + n1);
                    //System.out.println("n2     : " + n2);
                    //System.out.println("L      : " + L);
                    //System.out.println("fTable : " + fTable[0] + "  " + fTable[1] + "  " + fTable[2] + "  " + fTable[3]);
                    //System.out.println("Fext   : " + Fext[0] + "  " + Fext[1]);
                    //System.out.println("Fint   : " + Fint[0] + "  " + Fint[1] + "  " + Fint[2] + "  " + Fint[3]);
                    core.MaterialBeam beamMat=domain.getBeamMat();
                    core.MaterialSoil soilMat=domain.getSoilMat();
                    core.Section sec=domain.getBeamSec();
                    core.Node node1=domain.getNodes().get(n1-1);
                    core.Node node2=domain.getNodes().get(n2-1);
                    core.Beam b=new core.Beam(i, node1, node2, beamMat, soilMat, sec, fTable, Fext, Fint);
                    domain.addBeam(b);
                }
            }

        } catch (SAXParseException err) {
            System.out.println("** Parsing error" + ", line " + err.getLineNumber() + ", uri " + err.getSystemId());
            System.out.println(" " + err.getMessage());
            throw(new IOException());
        } catch (SAXException e) {
            Exception x = e.getException();
            ((x == null) ? e : x).printStackTrace();
            throw(new IOException());
        } catch (Throwable t) {
            t.printStackTrace();
            throw(new IOException());
        }

    }
}
